// Header for module xdos, generated by p2c 1.21alpha-07.Dec.93
#ifndef XDOS_H
#define XDOS_H


#ifdef XDOS_G
# define vextern
#else
# define vextern extern
#endif


struct SearchRec {
  int Attr;
  long Time, Size;
  Char Name[256];
};


#define AnyFile   1
#define Directory 2
#define Hidden    4
#define Archive   8

//extern Char *GetEnv(Char *, Char *);
#define GetEnv getenv

extern void FindFirst(Char *, int, SearchRec *SR);
extern void FindNext(SearchRec *SR);
extern void GetTime(int *hour, int *min, int *sec, int *hsec);
extern void GetDate(int *year, int *month, int *day, int *dow);
extern void FSplit(Char *,Char *,Char *,Char *);


extern long ReadTimer();
extern void StartTimer();
 
vextern int DosError;

#undef vextern

#endif /*XDOS_H*/

// End.
