// Header for module KeyConst, generated by p2c 1.21alpha-07.Dec.93
#ifndef KEYCONST_H
#define KEYCONST_H


#ifdef KEYCONST_G
# define vextern
#else
# define vextern extern
#endif



#define wsUpKey         '\005'   /* Ctrl-E */
#define wsDownKey       '\030'   /* Ctrl-X */
#define wsLeftKey       '\023'   /* Ctrl-S */
#define wsRightKey      '\004'   /* Ctrl-D */
#define wsDelKey        '\007'   /* Ctrl-G */
#define wsPgUpKey       '\022'   /* Ctrl-R */
#define wsPgDnKey       '\003'   /* Ctrl-C */
#define wsInsKey        '\026'   /* Ctrl-V */

#define wsBlockKey      '\013'   /* Ctrl-K */
#define wsQuickKey      '\021'   /* Ctrl-Q */
#define wsTab           '\t'   /* Ctrl-I */
#define wsBackSpace     '\b'

// #define fmAltF3Key      'j'

/*
#define fmAltWKey       '\021'
#define fmAltEKey       '\022'
#define fmAltRKey       '\023'
#define fmAltTKey       '\024'
#define fmAltUKey       '\026'
#define fmAltIKey       '\027'
#define fmAltXKey       '-'
*/

#define fmAltAKey	30
#define fmAltBKey	48
#define fmAltCKey	46
#define fmAltDKey	32
#define fmAltEKey	18
#define fmAltFKey	33
#define fmAltGKey	34
#define fmAltHKey	35
#define fmAltIKey	23
#define fmAltJKey	36
#define fmAltKKey	37
#define fmAltLKey	38
#define fmAltMKey	50
#define fmAltNKey	49
#define fmAltOKey	24
#define fmAltPKey	25
#define fmAltQKey	16
#define fmAltRKey	19
#define fmAltSKey	31
#define fmAltTKey	20
#define fmAltUKey	22
#define fmAltVKey	47
#define fmAltWKey	17
#define fmAltXKey	45
#define fmAltYKey	44
#define fmAltZKey	21


#define fmInsKey        'R'
#define fmDelKey        'S'
#define fmPos1Key       'G'
#define fmEndKey        'O'
#define fmPgUpKey       'I'
#define fmPgDnKey       'Q'
#define fmUpKey         'H'
#define fmDownKey       'P'
#define fmLeftKey       'K'
#define fmRightKey      'M'


// #define fmAltF10Key     'q'

#define fmCtrlLeftKey   's'
#define fmCtrlRightKey  't'


#define fmF1Key         ';'
#define fmF2Key         '<'
#define fmF3Key         '='
#define fmF4Key         '>'
#define fmF5Key         '?'
#define fmF6Key         '@'
#define fmF7Key         'A'
#define fmF8Key         'B'
#define fmF9Key         'C'
#define fmF10Key        'D'
#define fmF11Key        'E'
#define fmF12Key        'F'


#define	fmAltF1Key	104
#define	fmAltF2Key	105
#define	fmAltF3Key	106
#define	fmAltF4Key	107
#define	fmAltF5Key	108
#define	fmAltF6Key	109
#define	fmAltF7Key	110
#define	fmAltF8Key	111
#define	fmAltF9Key	112
#define	fmAltF10Key	113
#define	fmAltF11Key	114
#define	fmAltF12Key	115
#define	fmCtrlF1Key	94
#define	fmCtrlF2Key	95
#define	fmCtrlF3Key	96
#define	fmCtrlF4Key	97
#define	fmCtrlF5Key	98
#define	fmCtrlF6Key	99
#define	fmCtrlF7Key	100
#define	fmCtrlF8Key	101
#define	fmCtrlF9Key	102
#define	fmCtrlF10Key	103
#define	fmCtrlF11Key	104
#define	fmCtrlF12Key	105




//-----------------------------------------------------------------

#define evKeyUp         0x1000
#define evKeyDown       0x1001
#define evKeyLeft       0x1002
#define evKeyRight      0x1003
#define evKeyPgUp       0x1004
#define evKeyPgDn       0x1005
#define evKeyPos1       0x1006
#define evKeyEnd        0x1007
#define evKeyIns        0x1008
#define evKeyDel        0x1009
#define evKeyEnter      0x100a
#define evKeyAnyKey     0x100b

#define evKeyAltX       0x100c

#define evRightButton   0x100d
#define evLeftButton    0x100e
#define evBothButton    0x100f
#define evMiddleButton  0x1010

#define evKeyF1         0x1011
#define evKeyF2         0x1012
#define evKeyF3         0x1013
#define evKeyF4         0x1014
#define evKeyF5         0x1015
#define evKeyF6         0x1016
#define evKeyF7         0x1017
#define evKeyF8         0x1018
#define evKeyF9         0x1019
#define evKeyF10        0x101a
#define evKeyF11        0x101b
#define evKeyF12        0x101c


#undef vextern

#endif /*KEYCONST_H*/

// End.
