// Header for module fexist, generated by p2c 1.21alpha-07.Dec.93
#ifndef FEXIST_H
#define FEXIST_H


#ifdef FEXIST_G
# define vextern
#else
# define vextern extern
#endif

extern boolean FileExists(char *);


#undef vextern

#endif /*FEXIST_H*/

// End.
