// Header for module NCRT, generated by p2c 1.21alpha-07.Dec.93
#ifndef NCRT_H
#define NCRT_H


#ifdef NCRT_G
# define vextern
#else
# define vextern extern
#endif





#include <curses.h>
#include <unistd.h>
#include <stdio.h>



#define EGA_black       0
#define EGA_blue        1
#define EGA_green       2
#define EGA_cyan        3
#define EGA_red         4
#define EGA_magenta     5
#define EGA_brown       20
#define EGA_lightgray   7
#define EGA_darkgray    56
#define EGA_lightblue   57
#define EGA_lightgreen  58
#define EGA_lightcyan   59
#define EGA_lightred    60
#define EGA_lightmagenta  61
#define EGA_yellow      62
#define EGA_white       63


extern uchar EGA_Colors[16];
extern unsigned char Letters_Map[26];


vextern unsigned short WindMin, WindMax, WhereXY;
vextern uchar TextAttr;
vextern unsigned long int cur_color,cur_attr;

extern void Window(int, int, int, int);
extern void GotoXY(int, int);
extern unsigned short WhereX(void);
extern unsigned short WhereY(void);

extern void ClrScr(void);
extern void ClrEol(void);
extern void ClrEof(void);

extern void TextColor(uchar);
extern void TextBackGround(uchar);

extern void Sound(unsigned short);
extern void NoSound(void);
extern void Delay(unsigned short);

extern char ReadKey(void);
extern boolean KeyPressed(void);



extern void _CRT_Init(void);
extern void _CRT_Done(void);
extern void _Set_Window(void);
extern void _Set_Attribute(void);

extern unsigned long int ct_attr(unsigned char); 


#undef vextern

#endif /*NCRT_H*/

// End.
