// Output from p2c 1.21alpha-07.Dec.93, the Pascal-to-C translator
// From input file "tversion.p"


//$N-,F+,O+



#include <p2c/p2c.h>


#define TVERSION_G
#include <p2c/tversion.h>


boolean RunErrorInfo = false;


void IntroVersion(Char *, Char *, Char *, uchar)
{
}

void CryptVersion(Char *, Char *, Char *)
{
}

void Version(Char *, Char *)
{
}

void EnterProgSelfPath(Char *st)
{
  printf("%s \n written by Thomas Pantzer\n",st);
  sprintf(ProgSelfPath,"./");
}



