// Header for module ExitHandler, generated by p2c 1.21alpha-07.Dec.93
#ifndef EXITHANDLER_H
#define EXITHANDLER_H


#ifdef EXITHANDLER_G
# define vextern
#else
# define vextern extern
#endif




typedef void (*ProcType)(void);


class ExitProcType {
public:
  ProcType proc;
  long link;

  void Halt(void);
};


vextern ExitProcType ExitProc;


#undef vextern

#endif /*EXITHANDLER_H*/

// End.
