// Header for module TVersion, generated by p2c 1.21alpha-07.Dec.93
#ifndef TVERSION_H
#define TVERSION_H


#ifdef TVERSION_G
# define vextern
#else
# define vextern extern
#endif


extern boolean RunErrorInfo;
vextern Char ProgSelfPath[80];


extern void IntroVersion(Char *, Char *, Char *, uchar);
extern void CryptVersion(Char *, Char *, Char *);
extern void Version(Char *, Char *);
extern void EnterProgSelfPath(Char *);


#undef vextern

#endif /*TVERSION_H*/

// End.
