// Header for module Rs232B, generated by p2c 1.21alpha-07.Dec.93
#ifndef RS232B_H
#define RS232B_H


#ifdef RS232B_G
# define vextern
#else
# define vextern extern
#endif


extern unsigned short BaseAddr;
extern unsigned short StatusAddr;
extern unsigned short TimeOut;
extern boolean HandShake;
extern boolean XONOFF;
extern boolean XCtrl;
extern unsigned short MaxTimeOut;


extern void SetBaud(unsigned short);
extern void SetLocal(uchar, uchar, uchar);
extern void ReadPreSet(unsigned short &, uchar &, uchar &, uchar &);
extern void InstallRs232Exit(uchar);


class Int14Type {
public:
  uchar PortNr;
  boolean LastError;
  uchar CommStatus;

  void Init(uchar, uchar, uchar, uchar, uchar);
  void Send(uchar);
  void CheckError(void);
  uchar Receive(void);
  uchar Getc(void);
  unsigned short Status(void);
  void Done(void);
};


vextern Int14Type *Int14Call;


#undef vextern

#endif /*RS232B_H*/

// End.
