// Header for module StrType, generated by p2c 1.21alpha-07.Dec.93
#ifndef STRTYPE_H
#define STRTYPE_H


#ifdef STRTYPE_G
# define vextern
#else
# define vextern extern
#endif


typedef Char Str3[4];
typedef Char Str4[5];
typedef Char Str6[7];
typedef Char Str8[9];
typedef Char Str12[13];
typedef Char Str20[21];
typedef Char Str24[25];
typedef Char Str30[31];
typedef Char Str40[41];
typedef Char Str55[56];
typedef Char Str79[128];
typedef Char Str120[121];

typedef Char ExtStr[40];
typedef Char NameStr[80];
typedef Char DirStr[128];
typedef Char PathStr[256];


extern Char *GetWord(Char *, Char *, int);
extern Char *FormatHex(Char *, Char *);

extern Char *WToString(Char *, unsigned short);
extern Char *IToString(Char *, int);
extern Char *BToString(Char *, uchar);
extern Char *BToUFHex(Char *, uchar);
extern Char *BToHex(Char *, uchar);
extern Char *WToHex(Char *, unsigned short);
extern Char *BToFString(Char *, uchar, uchar);
extern Char *BToNString(Char *, uchar, uchar);
extern Char *WToFString(Char *, unsigned short);
extern Char *SToFString(Char *, short);
extern Char *LToString(Char *, long);
extern Char *LToFString(Char *, long, uchar);
extern Char *RToString(Char *, float);
extern Char *RToFString(Char *, float, uchar, uchar);
extern Char *BoolToString(Char *, boolean);
extern Char *BToBin(Char *, unsigned short);

extern int ToInteger(Char *);
extern uchar ToByte(Char *);
extern unsigned short ToWord(Char *);
extern float ToReal(Char *);
extern double ToExtended(Char *);


#undef vextern

#endif /*STRTYPE_H*/

// End.
