// Output from p2c 1.21alpha-07.Dec.93, the Pascal-to-C translator
// From input file "fexist.p"


#include <p2c/p2c.h>


#define FEXIST_G
#include "/software/include/p2c/fexist.h"


boolean FileExists(char *FileName)
{
  FILE *f = fopen(FileName, "r+b");
  _SETIO(f != NULL, FileNotFound);
  if (f != NULL)
    fclose(f);
  boolean __Result__ = P_ioresult == 0 && *FileName != '\0';
  return __Result__;
}


void _fexist_init(void)
{
  static int _was_initialized = 0;
  if (_was_initialized++)
    return;
}
// p2c: Note: Remember to call _fexist_init() in main program [215]



// End.
