// Output from p2c 1.21alpha-07.Dec.93, the Pascal-to-C translator
// From input file "/software/include/p2c/import/exitproc.p"


#include <p2c/p2c.h>


#define EXITHANDLER_G
#include "/software/include/p2c/exithandler.h"


void ExitProcType::Halt(void)
{
  ExitProcType *Next;

  proc();
// p2c: /software/include/p2c/import/exitproc.p, line 30:
// * Warning: No field called PROC in that record [288]

  if (link != 0)
     {
      Next=(ExitProcType *)link;
      Next->Halt();
     } 
}



void _ExitHandler_init(void)
{
  static int _was_initialized = 0;
  if (_was_initialized++)
    return;
}
// p2c: Note: Remember to call _ExitHandler_init() in main program [215]




// End.
